// temperature sensor includes
#include <OneWire.h>
#include <DallasTemperature.h>

// wifi/osc includes
#include <WiFi.h>
#include <WiFiUdp.h>
#include <OSCMessage.h>

// Sensor input pin
#define DATA_PIN 12

// SMD pins
#define LED_PIN_R 14
#define LED_PIN_G 27
#define LED_PIN_B 26


// Temp sensor stuff
// How many bits to use for temperature values: 9, 10, 11 or 12
#define SENSOR_RESOLUTION 10
// Index of sensors connected to data pin, default: 0
#define SENSOR_INDEX 0

// Temperature sensor initialization
OneWire oneWire(DATA_PIN);
DallasTemperature sensors(&oneWire);
DeviceAddress sensorDeviceAddress;

// WIFI/OSC stuff
WiFiUDP Udp; // A UDP instance to let us send and receive packets over UDP

char ssid[] = "Rogiana"; // your network SSID (name)
char pass[] = "CSeH45H0nmfd2934Gd!"; // your network password

const IPAddress outIp(10,0,0,9); // remote IP of your computer
const unsigned int outPort = 45678; // remote port to receive OSC
const unsigned int localPort = 8888; // local port to listen for OSC packets

// PWM stuff
const int ledChannel_r = 0;
const int ledChannel_g = 1;
const int ledChannel_b = 2;
const int resolution = 8;

// SMD values coming from OSC
int r;
int g;
int b;

// Global led control
bool led_on = false;

void setup() {
  Serial.begin(115200);

  // Sensor init
  sensors.begin();
  sensors.getAddress(sensorDeviceAddress, 0);
  sensors.setResolution(sensorDeviceAddress, SENSOR_RESOLUTION);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  WiFi.begin(ssid, pass);

  while (WiFi.status() != WL_CONNECTED) {
      delay(500);
      Serial.print(".");
  }
  Serial.println("");

  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

  Serial.println("Starting UDP");
  Udp.begin(localPort);
  Serial.print("Local port: ");
#ifdef ESP32
  Serial.println(localPort);
#else
  Serial.println(Udp.localPort());
#endif

// Configure PWMs
  ledcSetup(ledChannel_r, 5000, resolution);
  ledcSetup(ledChannel_g, 5000, resolution);
  ledcSetup(ledChannel_b, 5000, resolution);
  
  // attach the channel to the GPIO to be controlled
  ledcAttachPin(LED_PIN_R, ledChannel_r);
  ledcAttachPin(LED_PIN_G, ledChannel_g);
  ledcAttachPin(LED_PIN_B, ledChannel_b);
}


// Global led control
void led(OSCMessage &msg) {  
  int ledState = round(msg.getFloat(0));
  Serial.println("T msg: " + String(ledState));
  
  if(ledState == 1)
  {
    led_on = true;
  }
  else
  {
    led_on = false;
  }
}

void ledR(OSCMessage &msg) {  
  r = round(msg.getFloat(0)*255.0);
  //Serial.println("R msg: " + String(r));
}

void ledG(OSCMessage &msg) {  
  g = round(msg.getFloat(0)*255.0);
  //Serial.println("G msg: " + String(g));
}

void ledB(OSCMessage &msg) {  
  b = round(msg.getFloat(0)*255.0);
  //Serial.println("B msg: " + String(b));
}

void loop() {

  // Get temperature and process it
  sensors.requestTemperatures();  

  float temperatureInCelsius = sensors.getTempCByIndex(SENSOR_INDEX);
  //float temperatureInFahrenheit = sensors.getTempFByIndex(SENSOR_INDEX);

  //Serial.print("Temperature: ");
  //Serial.print(temperatureInCelsius, 4);
  //Serial.print(" Celsius, ");
  //Serial.println("");

  // Send the value to hololens
  OSCMessage msg("/fromboard");
  msg.add(temperatureInCelsius);
  Udp.beginPacket(outIp, outPort);
  msg.send(Udp);
  Udp.endPacket();
  msg.empty();

  // Receive led values from hololens UI
  OSCMessage msgr;
  int size = Udp.parsePacket();

  if (size > 0) {
    while (size--) {
      msgr.fill(Udp.read());
    }
    if (!msgr.hasError()) {
      msgr.dispatch("/toboard_r", ledR);
      msgr.dispatch("/toboard_g", ledG);
      msgr.dispatch("/toboard_b", ledB);
      msgr.dispatch("/toboard_toggle", led);
    } else {
      OSCErrorCode error = msgr.getError();
      Serial.print("error: ");
      Serial.println(error);
    }
  }

  // Control SMD lighting
  if( led_on )
  {
    ledcWrite(ledChannel_r, r);
    ledcWrite(ledChannel_g, g);
    ledcWrite(ledChannel_b, b);
  }
  else
  {
    ledcWrite(ledChannel_r, 0);
    ledcWrite(ledChannel_g, 0);
    ledcWrite(ledChannel_b, 0);
  }
}
